@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -42,7 +42,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <boost/program_options.hpp>
 
 #include <iostream>
-#include <iterator>
 #include <string>
 #include <utility>
 #include <vector>
@@ -177,6 +176,9 @@ void CommandAddLocationsToWays::find_member_nodes() {
 }
 
 bool CommandAddLocationsToWays::run() {
+    m_output_file.set("locations_on_ways");
+    osmium::io::Writer writer{m_output_file, m_output_overwrite, m_fsync};
+
     if (m_keep_member_nodes) {
         m_vout << "Getting all nodes referenced from relations...\n";
         find_member_nodes();
@@ -192,14 +194,12 @@ bool CommandAddLocationsToWays::run() {
         location_handler.ignore_errors();
     }
 
-    m_output_file.set("locations_on_ways");
-
     if (m_input_files.size() == 1) { // single input file
         m_vout << "Copying input file '" << m_input_files[0].filename() << "'...\n";
         osmium::io::Reader reader{m_input_files[0]};
         osmium::io::Header header{reader.header()};
         setup_header(header);
-        osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
+        writer.set_header(header);
 
         osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
         copy_data(progress_bar, reader, writer, location_handler);
@@ -210,7 +210,7 @@ bool CommandAddLocationsToWays::run() {
     } else { // multiple input files
         osmium::io::Header header;
         setup_header(header);
-        osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
+        writer.set_header(header);
 
         osmium::ProgressBar progress_bar{file_size_sum(m_input_files), display_progress()};
         for (const auto& input_file : m_input_files) {
