@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <boost/program_options.hpp>
 
 #include <cassert>
+#include <cstdlib>
 #include <string>
 #include <utility>
 #include <vector>
@@ -131,7 +132,9 @@ void CommandCat::write_buffers(osmium::ProgressBar& progress_bar, std::vector<os
     }
 }
 
-static void report_filename(osmium::VerboseOutput* vout, const osmium::io::File& file, const osmium::io::Reader& reader) {
+namespace {
+
+void report_filename(osmium::VerboseOutput* vout, const osmium::io::File& file, const osmium::io::Reader& reader) {
     assert(vout);
 
     const auto size = reader.file_size();
@@ -148,6 +151,8 @@ static void report_filename(osmium::VerboseOutput* vout, const osmium::io::File&
     }
 }
 
+} // anonymous namespace
+
 bool CommandCat::run() {
     std::size_t bytes_written = 0;
 
