@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -200,6 +200,8 @@ void CommandChangesetFilter::show_arguments() {
     }
 }
 
+namespace {
+
 bool changeset_after(const osmium::Changeset& changeset, osmium::Timestamp time) {
     return changeset.open() || changeset.closed_at() >= time;
 }
@@ -208,6 +210,8 @@ bool changeset_before(const osmium::Changeset& changeset, osmium::Timestamp time
     return changeset.created_at() <= time;
 }
 
+} // anonymous namespace
+
 bool CommandChangesetFilter::run() {
     m_vout << "Opening input file...\n";
     osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::changeset};
