@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <algorithm>
 #include <cassert>
 #include <cstdint>
+#include <cstdlib>
 #include <iostream>
 #include <string>
 #include <utility>
@@ -274,7 +275,7 @@ public:
                m_idset_neg(osmium::item_type::node).used_memory() +
                m_idset_neg(osmium::item_type::way).used_memory() +
                m_idset_neg(osmium::item_type::relation).used_memory() +
-               m_relation_refs.capacity() * sizeof(decltype(m_relation_refs)::value_type);
+               (m_relation_refs.capacity() * sizeof(decltype(m_relation_refs)::value_type));
     }
 
 }; // class RefCheckHandler
