@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 #include "export/export_format_json.hpp"
 #include "export/export_format_pg.hpp"
-#include "export/export_format_spaten.hpp"
 #include "export/export_format_text.hpp"
 #include "export/export_handler.hpp"
 
@@ -43,9 +42,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/util/verbose_output.hpp>
 #include <osmium/visitor.hpp>
 
-#include <rapidjson/document.h>
-#include <rapidjson/error/en.h>
-#include <rapidjson/istreamwrapper.h>
+#include <nlohmann/json.hpp>
 
 #include <boost/program_options.hpp>
 
@@ -57,27 +54,100 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <utility>
 #include <vector>
 
-static std::string get_attr_string(const rapidjson::Value& object, const char* key) {
-    const auto it = object.FindMember(key);
-    if (it == object.MemberEnd()) {
-        return "";
+namespace {
+
+std::string get_attr_string(const nlohmann::json& object, const char* key) {
+    const auto it = object.find(key);
+    if (it == object.end()) {
+        return {};
     }
 
-    if (it->value.IsString()) {
-        return it->value.GetString();
+    if (it->is_string()) {
+        return it->template get<std::string>();
     }
 
-    if (it->value.IsBool()) {
-        if (it->value.GetBool()) {
-            return std::string{"@"} + key;
+    if (it->is_boolean() && it->template get<bool>()) {
+        return std::string{"@"} + key;
+    }
+
+    return {};
+}
+
+Ruleset parse_tags_ruleset(const nlohmann::json& object, const char* key) {
+    Ruleset ruleset;
+
+    const auto json = object.find(key);
+    if (json == object.end() || json->is_null()) {
+        // When this is not set, the default is "other". This is later
+        // changed to "any" if both linear_tags and area_tags are missing.
+        ruleset.set_rule_type(tags_filter_rule_type::other);
+        return ruleset;
+    }
+
+    if (json->is_boolean()) {
+        if (json->template get<bool>()) {
+            ruleset.set_rule_type(tags_filter_rule_type::any);
+        } else {
+            ruleset.set_rule_type(tags_filter_rule_type::none);
+        }
+        return ruleset;
+    }
+
+    if (!json->is_array()) {
+        throw config_error{std::string{"'"} + key + "' member in top-level object must be false, true, null, or an array."};
+    }
+
+    if (json->empty()) {
+        std::cerr << "Warning! An empty array for 'linear_tags' or 'area_tags' matches any tags.\n"
+                  << "         Please use 'true' instead of the array.\n";
+        ruleset.set_rule_type(tags_filter_rule_type::any);
+        return ruleset;
+    }
+
+    ruleset.set_rule_type(tags_filter_rule_type::list);
+
+    for (const auto& value : *json) {
+        if (!value.is_string()) {
+            throw config_error{std::string{"Array elements in '"} + key + "' must be strings."};
+        }
+
+        const auto str = value.template get<std::string>();
+        if (!str.empty()) {
+            ruleset.add_rule(str);
         }
     }
 
-    return "";
+    return ruleset;
 }
 
-void CommandExport::parse_attributes(const rapidjson::Value& attributes) {
-    if (!attributes.IsObject()) {
+bool parse_string_array(const nlohmann::json& object, const char* key, std::vector<std::string>* result) {
+    const auto json = object.find(key);
+    if (json == object.end()) {
+        return false;
+    }
+
+    if (!json->is_array()) {
+        throw config_error{std::string{"'"} + key + "' member in top-level object must be array."};
+    }
+
+    for (const auto& value : *json) {
+        if (!value.is_string()) {
+            throw config_error{std::string{"Array elements in '"} + key + "' must be strings."};
+        }
+
+        const auto str = value.template get<std::string>();
+        if (!str.empty()) {
+            result->emplace_back(str);
+        }
+    }
+
+    return true;
+}
+
+} // anonymous namespace
+
+void CommandExport::parse_attributes(const nlohmann::json& attributes) {
+    if (!attributes.is_object()) {
         throw config_error{"'attributes' member must be an object."};
     }
 
@@ -91,133 +161,59 @@ void CommandExport::parse_attributes(const rapidjson::Value& attributes) {
     m_options.way_nodes = get_attr_string(attributes, "way_nodes");
 }
 
-void CommandExport::parse_format_options(const rapidjson::Value& options) {
-    if (!options.IsObject()) {
+void CommandExport::parse_format_options(const nlohmann::json& options) {
+    if (!options.is_object()) {
         throw config_error{"'format_options' member must be an object."};
     }
-    for (auto it = options.MemberBegin(); it != options.MemberEnd(); it++) {
-        const auto type = it->value.GetType();
-        const char* key = it->name.GetString();
+    for (const auto &item : options.items()) {
+        const auto type = item.value().type();
+        const auto &key = item.key();
         switch (type) {
-            case rapidjson::kNullType:
+            case nlohmann::json::value_t::null:
                 m_options.format_options.set(key, false);
                 break;
-            case rapidjson::kTrueType:
-                m_options.format_options.set(key, true);
+            case nlohmann::json::value_t::boolean:
+                m_options.format_options.set(key, item.value().template get<bool>());
                 break;
-            case rapidjson::kFalseType:
-                m_options.format_options.set(key, false);
-                break;
-            case rapidjson::kObjectType:
+            case nlohmann::json::value_t::object:
                 throw config_error{"Option value for key '" + std::string(key) + "' can not be of type object."};
-            case rapidjson::kArrayType:
+            case nlohmann::json::value_t::array:
                 throw config_error{"Option value for key '" + std::string(key) + "' can not be an array."};
                 break;
-            case rapidjson::kStringType:
-                m_options.format_options.set(key, it->value.GetString());
+            case nlohmann::json::value_t::string:
+                m_options.format_options.set(key, item.value().template get<std::string>());
                 break;
-            case rapidjson::kNumberType:
-                m_options.format_options.set(key, std::to_string(it->value.GetInt64()));
+            case nlohmann::json::value_t::number_integer:
+                m_options.format_options.set(key, std::to_string(item.value().template get<int64_t>()));
                 break;
+            case nlohmann::json::value_t::number_unsigned:
+                m_options.format_options.set(key, std::to_string(item.value().template get<uint64_t>()));
+                break;
+            case nlohmann::json::value_t::number_float:
+                m_options.format_options.set(key, std::to_string(item.value().template get<double>()));
+                break;
+            default:
+                throw config_error{"Unknown type"};
         }
     }
 }
 
-static Ruleset parse_tags_ruleset(const rapidjson::Value& object, const char* key) {
-    Ruleset ruleset;
-
-    const auto json = object.FindMember(key);
-    if (json == object.MemberEnd() || json->value.IsNull()) {
-        // When this is not set, the default is "other". This is later
-        // changed to "any" if both linear_tags and area_tags are missing.
-        ruleset.set_rule_type(tags_filter_rule_type::other);
-        return ruleset;
-    }
-
-    if (json->value.IsFalse()) {
-        ruleset.set_rule_type(tags_filter_rule_type::none);
-        return ruleset;
-    }
-
-    if (json->value.IsTrue()) {
-        ruleset.set_rule_type(tags_filter_rule_type::any);
-        return ruleset;
-    }
-
-    if (!json->value.IsArray()) {
-        throw config_error{std::string{"'"} + key + "' member in top-level object must be false, true, null, or an array."};
-    }
-
-    if (json->value.GetArray().Empty()) {
-        std::cerr << "Warning! An empty array for 'linear_tags' or 'area_tags' matches any tags.\n"
-                  << "         Please use 'true' instead of the array.\n";
-        ruleset.set_rule_type(tags_filter_rule_type::any);
-        return ruleset;
-    }
-
-    ruleset.set_rule_type(tags_filter_rule_type::list);
-
-    for (const auto& value : json->value.GetArray()) {
-        if (!value.IsString()) {
-            throw config_error{std::string{"Array elements in '"} + key + "' must be strings."};
-        }
-
-        if (value.GetString()[0] != '\0') {
-            ruleset.add_rule(value.GetString());
-        }
-    }
-
-    return ruleset;
-}
-
-static bool parse_string_array(const rapidjson::Value& object, const char* key, std::vector<std::string>* result) {
-    const auto json = object.FindMember(key);
-    if (json == object.MemberEnd()) {
-        return false;
-    }
-
-    if (!json->value.IsArray()) {
-        throw config_error{std::string{"'"} + key + "' member in top-level object must be array."};
-    }
-
-    for (const auto& value : json->value.GetArray()) {
-        if (!value.IsString()) {
-            throw config_error{std::string{"Array elements in '"} + key + "' must be strings."};
-        }
-
-        if (value.GetString()[0] != '\0') {
-            result->emplace_back(value.GetString());
-        }
-    }
-
-    return true;
-}
-
 void CommandExport::parse_config_file() {
     std::ifstream config_file{m_config_file_name};
-    rapidjson::IStreamWrapper stream_wrapper{config_file};
+    nlohmann::json doc = nlohmann::json::parse(config_file);
 
-    rapidjson::Document doc;
-    if (doc.ParseStream<(rapidjson::kParseCommentsFlag | rapidjson::kParseTrailingCommasFlag)>(stream_wrapper).HasParseError()) {
-        throw config_error{std::string{"JSON error at offset "} +
-                           std::to_string(doc.GetErrorOffset()) +
-                           ": " +
-                           rapidjson::GetParseError_En(doc.GetParseError())
-                          };
-    }
-
-    if (!doc.IsObject()) {
+    if (!doc.is_object()) {
         throw config_error{"Top-level value must be an object."};
     }
 
-    const auto json_attr = doc.FindMember("attributes");
-    if (json_attr != doc.MemberEnd()) {
-        parse_attributes(json_attr->value);
+    const auto json_attr = doc.find("attributes");
+    if (json_attr != doc.end()) {
+        parse_attributes(*json_attr);
     }
 
-    const auto json_opts = doc.FindMember("format_options");
-    if (json_opts != doc.MemberEnd()) {
-        parse_format_options(json_opts->value);
+    const auto json_opts = doc.find("format_options");
+    if (json_opts != doc.end()) {
+        parse_format_options(*json_opts);
     }
 
     m_linear_ruleset = parse_tags_ruleset(doc, "linear_tags");
@@ -355,9 +351,8 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     if (m_output_format != "geojson" &&
         m_output_format != "geojsonseq" &&
         m_output_format != "pg" &&
-        m_output_format != "text" &&
-        m_output_format != "spaten") {
-        throw argument_error{"Set output format with --output-format or -f to 'geojson', 'geojsonseq', 'pg', 'spaten', or 'text'."};
+        m_output_format != "text") {
+        throw argument_error{"Set output format with --output-format or -f to 'geojson', 'geojsonseq', 'pg', or 'text'."};
     }
 
     // Set defaults for output format options depending on output format
@@ -373,6 +368,10 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
 
         try {
             parse_config_file();
+        } catch (const nlohmann::json::parse_error& e) {
+            std::cerr << "Error while reading config file '" << m_config_file_name << "':\n";
+            throw config_error{std::string{"JSON error at offset "} +
+                               std::to_string(e.byte) + ": " + e.what()};
         } catch (const config_error&) {
             std::cerr << "Error while reading config file '" << m_config_file_name << "':\n";
             throw;
@@ -484,14 +483,16 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     return true;
 }
 
-static void print_taglist(osmium::VerboseOutput* vout, const std::vector<std::string>& strings) {
+namespace {
+
+void print_taglist(osmium::VerboseOutput* vout, const std::vector<std::string>& strings) {
     assert(vout);
     for (const auto& str : strings) {
         *vout << "    " << str << '\n';
     }
 }
 
-static void print_ruleset(osmium::VerboseOutput* vout, const Ruleset& ruleset) {
+void print_ruleset(osmium::VerboseOutput* vout, const Ruleset& ruleset) {
     assert(vout);
     switch (ruleset.rule_type()) {
         case tags_filter_rule_type::none:
@@ -510,7 +511,7 @@ static void print_ruleset(osmium::VerboseOutput* vout, const Ruleset& ruleset) {
     }
 }
 
-static const char* print_unique_id_type(unique_id_type unique_id) {
+const char* print_unique_id_type(unique_id_type unique_id) {
     switch (unique_id) {
         case unique_id_type::counter:
             return "counter";
@@ -523,6 +524,28 @@ static const char* print_unique_id_type(unique_id_type unique_id) {
     return "no";
 }
 
+std::unique_ptr<ExportFormat> create_handler(const std::string& output_format,
+                                             const std::string& output_filename,
+                                             osmium::io::overwrite overwrite,
+                                             osmium::io::fsync fsync,
+                                             const options_type& options) {
+    if (output_format == "geojson" || output_format == "geojsonseq") {
+        return std::make_unique<ExportFormatJSON>(output_format, output_filename, overwrite, fsync, options);
+    }
+
+    if (output_format == "pg") {
+        return std::make_unique<ExportFormatPg>(output_format, output_filename, overwrite, fsync, options);
+    }
+
+    if (output_format == "text") {
+        return std::make_unique<ExportFormatText>(output_format, output_filename, overwrite, fsync, options);
+    }
+
+    throw argument_error{"Unknown output format"};
+}
+
+} // anonymous namespace
+
 void CommandExport::show_arguments() {
     show_single_input_arguments(m_vout);
 
@@ -567,30 +590,6 @@ void CommandExport::show_arguments() {
     m_vout << "    keep untagged features: " << yes_no(m_options.keep_untagged);
 }
 
-static std::unique_ptr<ExportFormat> create_handler(const std::string& output_format,
-                                                    const std::string& output_filename,
-                                                    osmium::io::overwrite overwrite,
-                                                    osmium::io::fsync fsync,
-                                                    const options_type& options) {
-    if (output_format == "geojson" || output_format == "geojsonseq") {
-        return std::make_unique<ExportFormatJSON>(output_format, output_filename, overwrite, fsync, options);
-    }
-
-    if (output_format == "pg") {
-        return std::make_unique<ExportFormatPg>(output_format, output_filename, overwrite, fsync, options);
-    }
-
-    if (output_format == "text") {
-        return std::make_unique<ExportFormatText>(output_format, output_filename, overwrite, fsync, options);
-    }
-
-    if (output_format == "spaten") {
-        return std::make_unique<ExportFormatSpaten>(output_format, output_filename, overwrite, fsync, options);
-    }
-
-    throw argument_error{"Unknown output format"};
-}
-
 bool CommandExport::run() {
     auto handler = create_handler(m_output_format, m_output_filename, m_output_overwrite, m_fsync, m_options);
     if (m_vout.verbose()) {
@@ -613,7 +612,7 @@ bool CommandExport::run() {
 
     if (m_index_type_name == "none") {
         osmium::io::ReaderWithProgressBar reader{display_progress(), m_input_file};
-        osmium::apply(reader, check_order_handler, export_handler, mp_manager.handler([&export_handler](osmium::memory::Buffer&& buffer) {
+        osmium::apply(reader, check_order_handler, export_handler, mp_manager.handler([&export_handler](const osmium::memory::Buffer& buffer) {
             osmium::apply(buffer, export_handler);
         }));
         reader.close();
@@ -627,7 +626,7 @@ bool CommandExport::run() {
         }
 
         osmium::io::ReaderWithProgressBar reader{display_progress(), m_input_filename};
-        osmium::apply(reader, check_order_handler, location_handler, export_handler, mp_manager.handler([&export_handler](osmium::memory::Buffer&& buffer) {
+        osmium::apply(reader, check_order_handler, location_handler, export_handler, mp_manager.handler([&export_handler](const osmium::memory::Buffer& buffer) {
             osmium::apply(buffer, export_handler);
         }));
         reader.close();
