@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/index/map/all.hpp>
 #include <osmium/io/writer_options.hpp>
 
-#include <rapidjson/document.h>
+#include <nlohmann/json.hpp>
 
 #include <string>
 #include <vector>
@@ -63,8 +63,8 @@ class CommandExport : public CommandWithSingleOSMInput {
     bool m_stop_on_error = false;
 
     void canonicalize_output_format();
-    void parse_attributes(const rapidjson::Value& attributes);
-    void parse_format_options(const rapidjson::Value& options);
+    void parse_attributes(const nlohmann::json& attributes);
+    void parse_format_options(const nlohmann::json& options);
     void parse_config_file();
 
 public:
