@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -42,9 +42,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <cstddef>
 #include <fstream>
 #include <iostream>
-#include <stdexcept>
 #include <string>
-#include <utility>
 #include <vector>
 
 std::size_t CommandGetId::count_ids() const noexcept {
@@ -219,7 +217,9 @@ osmium::osm_entity_bits::type CommandGetId::get_needed_types() const {
     return types;
 }
 
-static void print_missing_ids(const char* type, const osmium::index::IdSetDense<osmium::unsigned_object_id_type>& set) {
+namespace {
+
+void print_missing_ids(const char* type, const osmium::index::IdSetDense<osmium::unsigned_object_id_type>& set) {
     if (set.empty()) {
         return;
     }
@@ -230,6 +230,8 @@ static void print_missing_ids(const char* type, const osmium::index::IdSetDense<
     std::cerr << '\n';
 }
 
+} // anonymous namespace
+
 void CommandGetId::mark_rel_ids(const osmium::index::RelationsMapIndex& rel_in_rel, osmium::unsigned_object_id_type parent_id) {
     rel_in_rel.for_each(parent_id, [&](osmium::unsigned_object_id_type member_id) {
         if (m_ids(osmium::item_type::relation).check_and_set(member_id)) {
@@ -331,6 +333,9 @@ void CommandGetId::find_referenced_objects() {
 }
 
 bool CommandGetId::run() {
+    m_vout << "Opening output file...\n";
+    osmium::io::Writer writer{m_output_file, m_output_overwrite, m_fsync};
+
     if (m_add_referenced_objects) {
         find_referenced_objects();
     }
@@ -338,11 +343,9 @@ bool CommandGetId::run() {
     m_vout << "Opening input file...\n";
     osmium::io::Reader reader{m_input_file, get_needed_types()};
 
-    m_vout << "Opening output file...\n";
     osmium::io::Header header{reader.header()};
     setup_header(header);
-
-    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
+    writer.set_header(header);
 
     m_vout << "Copying matching objects to output file...\n";
     osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
