@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -150,7 +150,9 @@ void CommandShow::show_arguments() {
 }
 
 #ifndef _WIN32
-static int execute_pager(const std::string& pager, bool with_color) {
+namespace {
+
+int execute_pager(const std::string& pager, bool with_color) {
     int pipefd[2];
     if (::pipe(pipefd) < 0) {
         throw std::system_error{errno, std::system_category(), "Could not run pager: pipe() call failed"};
@@ -186,6 +188,8 @@ static int execute_pager(const std::string& pager, bool with_color) {
 
     return pipefd[1];
 }
+
+} // anonymous namespace
 #endif
 
 bool CommandShow::run() {
