@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <boost/program_options.hpp>
 
 #include <algorithm>
+#include <cmath>
 #include <string>
 #include <utility>
 #include <vector>
@@ -76,10 +77,6 @@ bool CommandSort::setup(const std::vector<std::string>& arguments) {
     setup_input_files(vm);
     setup_output_file(vm);
 
-    if (vm.count("input-filenames")) {
-        m_filenames = vm["input-filenames"].as<std::vector<std::string>>();
-    }
-
     if (vm.count("strategy")) {
         m_strategy = vm["strategy"].as<std::string>();
         if (m_strategy != "simple" && m_strategy != "multipass") {
@@ -87,6 +84,14 @@ bool CommandSort::setup(const std::vector<std::string>& arguments) {
         }
     }
 
+    if (m_strategy == "multipass") {
+        if (std::any_of(m_input_filenames.cbegin(), m_input_filenames.cend(), [&](const std::string& name) {
+            return name == "-";
+        })) {
+            throw argument_error{"Can not read from STDIN when using multipass strategy"};
+        }
+    }
+
     return true;
 }
 
@@ -99,6 +104,8 @@ void CommandSort::show_arguments() {
 }
 
 bool CommandSort::run_single_pass() {
+    osmium::io::Writer writer{m_output_file, m_output_overwrite, m_fsync};
+
     std::vector<osmium::memory::Buffer> data;
     osmium::ObjectPointerCollection objects;
 
@@ -110,8 +117,8 @@ bool CommandSort::run_single_pass() {
 
     m_vout << "Reading contents of input files...\n";
     osmium::ProgressBar progress_bar{file_size_sum(m_input_files), display_progress()};
-    for (const std::string& file_name : m_filenames) {
-        osmium::io::Reader reader{file_name, osmium::osm_entity_bits::object};
+    for (const auto& file : m_input_files) {
+        osmium::io::Reader reader{file, osmium::osm_entity_bits::object};
         const osmium::io::Header header{reader.header()};
         bounding_box.extend(header.joined_boxes());
         while (osmium::memory::Buffer buffer = reader.read()) {
@@ -144,8 +151,7 @@ bool CommandSort::run_single_pass() {
     if (bounding_box) {
         header.add_box(bounding_box);
     }
-
-    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
+    writer.set_header(header);
 
     m_vout << "Sorting data...\n";
     objects.sort(osmium::object_order_type_id_version());
@@ -164,11 +170,13 @@ bool CommandSort::run_single_pass() {
 }
 
 bool CommandSort::run_multi_pass() {
+    osmium::io::Writer writer{m_output_file, m_output_overwrite, m_fsync};
+
     osmium::Box bounding_box;
 
     m_vout << "Reading input file headers...\n";
-    for (const std::string& file_name : m_filenames) {
-        osmium::io::Reader reader{file_name, osmium::osm_entity_bits::nothing};
+    for (const auto& file : m_input_files) {
+        osmium::io::Reader reader{file, osmium::osm_entity_bits::nothing};
         const osmium::io::Header header{reader.header()};
         bounding_box.extend(header.joined_boxes());
         reader.close();
@@ -180,8 +188,7 @@ bool CommandSort::run_multi_pass() {
     if (bounding_box) {
         header.add_box(bounding_box);
     }
-
-    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
+    writer.set_header(header);
 
     osmium::ProgressBar progress_bar{file_size_sum(m_input_files) * 3, display_progress()};
 
@@ -196,8 +203,8 @@ bool CommandSort::run_multi_pass() {
 
         m_vout << "Pass " << pass++ << "...\n";
         m_vout << "Reading contents of input files...\n";
-        for (const std::string& file_name : m_filenames) {
-            osmium::io::Reader reader{file_name, entity};
+        for (const auto& file : m_input_files) {
+            osmium::io::Reader reader{file, entity};
             const osmium::io::Header read_header{reader.header()};
             bounding_box.extend(read_header.joined_boxes());
             while (osmium::memory::Buffer buffer = reader.read()) {
