@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,8 +30,14 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/util/progress_bar.hpp>
 
 #include <algorithm>
+#include <cstdlib>
+#include <cstring>
 #include <fstream>
 #include <functional>
+#include <limits>
+#include <string>
+#include <utility>
+#include <vector>
 
 void CommandTagsCount::add_matcher(const std::string& expression) {
     bool has_value_matcher = false;
@@ -65,7 +71,9 @@ void CommandTagsCount::read_expressions_file(const std::string& file_name) {
     }
 }
 
-static sort_func_type get_sort_function(const std::string& sort_order) {
+namespace {
+
+sort_func_type get_sort_function(const std::string& sort_order) {
     static const std::pair<std::string, sort_func_type> sort_options[] = {
         { "count-desc", [](const element_type& a, const element_type& b){
                 if (a.count == b.count) {
@@ -100,6 +108,8 @@ static sort_func_type get_sort_function(const std::string& sort_order) {
     throw argument_error{"Unknown sort order '" + sort_order + "'"};
 }
 
+} // anonymous namespace
+
 bool CommandTagsCount::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_cmd{"COMMAND OPTIONS"};
     opts_cmd.add_options()
@@ -216,7 +226,9 @@ std::vector<element_type> CommandTagsCount::sort_results() const {
     return results;
 }
 
-static void append_escaped(std::string* out, const char* str) {
+namespace {
+
+void append_escaped(std::string* out, const char* str) {
     *out += '"';
     for (; *str != '\0'; ++str) {
         if (*str == '"') {
@@ -227,7 +239,7 @@ static void append_escaped(std::string* out, const char* str) {
     *out += '"';
 }
 
-static void write_results(const std::vector<element_type>& results, int fd) {
+void write_results(const std::vector<element_type>& results, int fd) {
     std::string out;
     const std::size_t buffer_size = 1024UL * 1024UL;
     out.reserve(buffer_size);
@@ -253,6 +265,8 @@ static void write_results(const std::vector<element_type>& results, int fd) {
     close(fd);
 }
 
+} // anonymous namespace
+
 bool CommandTagsCount::run() {
     m_vout << "Opening input file...\n";
     osmium::io::Reader reader{m_input_file, osm_entity_bits(), osmium::io::read_meta::no};
