@@ -87,15 +87,15 @@ void ExportFormatJSON::start_feature(const std::string& prefix, osmium::object_i
         m_buffer += R"(,"id":)";
         m_buffer += std::to_string(m_count + 1);
     } else if (options().unique_id == unique_id_type::type_id) {
-        m_buffer += R"(,"id":)";
+        m_buffer += R"(,"id":")";
         m_buffer += prefix;
         m_buffer += std::to_string(id);
+        m_buffer += '"';
     }
 }
 
 void ExportFormatJSON::add_option(const std::string& name) {
     const nlohmann::json j = name;
-    m_buffer += ',';
     m_buffer += j.dump();
     m_buffer += ':';
 }
@@ -104,38 +104,47 @@ void ExportFormatJSON::add_attributes(const osmium::OSMObject& object) {
 
     if (!options().type.empty()) {
         add_option(options().type);
+        m_buffer += '"';
         m_buffer += object_type_as_string(object);
+        m_buffer += '"';
+        m_buffer += ',';
     }
 
     if (!options().id.empty()) {
         add_option(options().id);
         m_buffer += std::to_string(object.type() == osmium::item_type::area ? osmium::area_id_to_object_id(object.id()) : object.id());
+        m_buffer += ',';
     }
 
     if (!options().version.empty()) {
         add_option(options().version);
         m_buffer += std::to_string(object.version());
+        m_buffer += ',';
     }
 
     if (!options().changeset.empty()) {
         add_option(options().changeset);
         m_buffer += std::to_string(object.changeset());
+        m_buffer += ',';
     }
 
     if (!options().uid.empty()) {
         add_option(options().uid);
         m_buffer += std::to_string(object.uid());
+        m_buffer += ',';
     }
 
     if (!options().user.empty()) {
         add_option(options().user);
         const nlohmann::json j = object.user();
-        m_buffer += j.template get<std::string>();
+        m_buffer += j.dump();
+        m_buffer += ',';
     }
 
     if (!options().timestamp.empty()) {
         add_option(options().timestamp);
         m_buffer += std::to_string(object.timestamp().seconds_since_epoch());
+        m_buffer += ',';
     }
 
     if (!options().way_nodes.empty() && object.type() == osmium::item_type::way) {
@@ -151,6 +160,7 @@ void ExportFormatJSON::add_attributes(const osmium::OSMObject& object) {
         } else {
             m_buffer += ']';
         }
+        m_buffer += ',';
     }
 }
 
@@ -240,19 +250,18 @@ void ExportFormatJSON::create_linestring(const osmium::Way& way) {
 
 void ExportFormatJSON::create_multipolygon(const osmium::Area& area) {
     m_buffer += R"(,"geometry":{"type":"MultiPolygon","coordinates":)";
+    m_buffer += '[';
     for (const auto &outer_ring : area.outer_rings()) {
-        m_buffer += "[[";
+        m_buffer += '[';
         create_coordinate_list(outer_ring);
-        m_buffer += ']';
         for (const auto &inner_ring : area.inner_rings(outer_ring)) {
-            m_buffer += ",[";
+            m_buffer += ",";
             create_coordinate_list(inner_ring);
-            m_buffer += ']';
         }
         m_buffer += "],";
     }
-
-    m_buffer.back() = '}';
+    m_buffer.pop_back(); // remove trailing comma
+    m_buffer += "]}";
 }
 
 void ExportFormatJSON::node(const osmium::Node& node) {
