@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -28,13 +28,8 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/io/detail/read_write.hpp>
 #include <osmium/io/detail/string_util.hpp>
 
-#ifndef RAPIDJSON_HAS_STDSTRING
-# define RAPIDJSON_HAS_STDSTRING 1
-#endif
-#include <rapidjson/stringbuffer.h>
-#include <rapidjson/writer.h>
+#include <nlohmann/json.hpp>
 
-#include <limits>
 #include <string>
 
 enum {
@@ -83,8 +78,8 @@ void ExportFormatPg::start_feature(const char type, const osmium::object_id_type
     }
 }
 
-void ExportFormatPg::append_pg_escaped(const char* str, std::size_t size = std::numeric_limits<std::size_t>::max()) {
-    while (size-- > 0 && *str != '\0') {
+void ExportFormatPg::append_pg_escaped(const char* str) {
+    while (*str != '\0') {
         switch (*str) {
             case '\\':
                 m_buffer += '\\';
@@ -166,27 +161,36 @@ void ExportFormatPg::add_attributes(const osmium::OSMObject& object) {
 }
 
 bool ExportFormatPg::add_tags_json(const osmium::OSMObject& object) {
-    bool has_tags = false;
+    std::string target{"{"};
+    nlohmann::json j;
 
-    rapidjson::StringBuffer stream;
-    rapidjson::Writer<rapidjson::StringBuffer> writer{stream};
-
-    writer.StartObject();
     for (const auto& tag : object.tags()) {
         if (options().tags_filter(tag)) {
-            has_tags = true;
-            writer.Key(tag.key());
-            writer.String(tag.value());
+            j = tag.key();
+            target += j.dump();
+            target += ':';
+            j = tag.value();
+            target += j.dump();
+            target += ',';
         }
     }
-    writer.EndObject();
 
-    append_pg_escaped(stream.GetString(), stream.GetSize());
+    bool const has_tags = target.size() > 1;
+
+    if (has_tags) {
+        target.back() = '}';
+    } else {
+        target += '}';
+    }
+
+    append_pg_escaped(target.c_str());
 
     return has_tags;
 }
 
-static void add_escape_hstore(std::string* out, const char* str) {
+namespace {
+
+void add_escape_hstore(std::string* out, const char* str) {
     *out += "\"";
 
     while (*str) {
@@ -203,6 +207,8 @@ static void add_escape_hstore(std::string* out, const char* str) {
     *out += "\"";
 }
 
+} // anonymous namespace
+
 bool ExportFormatPg::add_tags_hstore(const osmium::OSMObject& object) {
     if (object.tags().empty()) {
         return false;
