@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ class ExportFormatPg : public ExportFormat {
     bool add_tags_hstore(const osmium::OSMObject& object);
     bool add_tags(const osmium::OSMObject& object);
     void finish_feature(const osmium::OSMObject& object);
-    void append_pg_escaped(const char* str, std::size_t size);
+    void append_pg_escaped(const char* str);
 
 public:
 
