@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 #include <cstddef>
 #include <string>
+#include <utility>
 
 using index_type = osmium::index::map::SparseMemArray<osmium::unsigned_object_id_type, osmium::Location>;
 using location_handler_type = osmium::handler::NodeLocationsForWays<index_type>;
@@ -63,7 +64,7 @@ std::size_t OSMFileParser::operator()() {
         osmium::builder::AreaBuilder builder{m_buffer};
 
         osmium::io::Reader reader{input_file};
-        osmium::apply(reader, location_handler, collector.handler([&](osmium::memory::Buffer&& buffer) {
+        osmium::apply(reader, location_handler, collector.handler([&](const osmium::memory::Buffer& buffer) {
             for (const auto& area : buffer.select<osmium::Area>()) {
                 for (const auto& item : area) {
                     if (item.type() == osmium::item_type::outer_ring ||
