@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2023  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2025  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,6 +27,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/handler/check_order.hpp>
 #include <osmium/util/file.hpp>
 
+#include <cstdlib>
+#include <memory>
+#include <vector>
+
 namespace strategy_complete_ways {
 
     void Data::add_relation_parents(osmium::unsigned_object_id_type id, const osmium::index::RelationsMapIndex& map) {
